@@ -90,6 +90,7 @@
 #' @export
 Collocations = R6::R6Class(
   private = list(
+    iter = NULL,
     sep = NULL,
     phrases = NULL,
     phrases_ptr = NULL,
@@ -110,7 +111,7 @@ Collocations = R6::R6Class(
                           llr_min = 0,
                           sep = "_") {
       if(is.null(vocabulary)) {
-        flog.debug("got NULL as vocabulary - so it will be built from training data iterator later")
+        logger$debug("got NULL as vocabulary - so it will be built from training data iterator later")
       } else if(inherits(vocabulary, "text2vec_vocabulary")) {
         private$vocabulary = copy(vocabulary)
       } else {
@@ -118,6 +119,7 @@ Collocations = R6::R6Class(
       }
 
       private$sep = sep
+      private$iter = 0L
       private$collocation_count_min = collocation_count_min
       private$pmi_min = pmi_min
       private$gensim_min = gensim_min
@@ -125,14 +127,16 @@ Collocations = R6::R6Class(
       private$llr_min = llr_min
     },
     fit = function(it, n_iter = 1L, ...) {
+      private$iter = 0L
       n_colloc_last = -1L
       for(i in seq_len(n_iter)) {
         self$partial_fit(it, ...)
         if(nrow(self$collocation_stat) > n_colloc_last) {
-          flog.info("iteration %d - found %d collocations", i, nrow(self$collocation_stat))
+          logger$info("iteration %d - found %d collocations", i, nrow(self$collocation_stat))
+          logger$debug("iteration %d - n_words = %d", i, attr(self$collocation_stat, "nword"))
           n_colloc_last = nrow(self$collocation_stat)
         } else {
-          flog.info("iteration %d - converged", i)
+          logger$info("iteration %d - converged", i)
           break()
         }
       }
@@ -141,10 +145,10 @@ Collocations = R6::R6Class(
     partial_fit = function(it, ...) {
       stopifnot(inherits(it, "itoken") || inherits(it, "itoken_parallel"))
       if(is.null(private$vocabulary)) {
-        flog.debug("building vocabulary for Collocations model")
+        logger$debug("building vocabulary for Collocations model")
         private$vocabulary = create_vocabulary(it, ...)
         private$vocabulary = prune_vocabulary(private$vocabulary, term_count_min = private$collocation_count_min)
-        flog.debug("vocabulary construction done - %d terms", nrow(private$vocabulary))
+        logger$debug("vocabulary construction done - %d terms", nrow(private$vocabulary))
       }
       if(!is.null(self$collocation_stat)) {
         private$vocabulary = create_vocabulary(it = unique(c(private$phrases, private$vocabulary$term)),
@@ -155,7 +159,7 @@ Collocations = R6::R6Class(
       }
       else {
         if(inherits(it, "itoken_parallel")) {
-          flog.debug("clonning itoken_parallel")
+          logger$debug("clonning itoken_parallel")
           it_internal = lapply(it, function(x) x$clone(deep = TRUE))
           data.table::setattr(it_internal, "class", "itoken_parallel")
         } else
@@ -164,7 +168,6 @@ Collocations = R6::R6Class(
       vectorizer = vocab_vectorizer(private$vocabulary)
       tcm = create_tcm(it_internal, vectorizer, skip_grams_window = 1L,
                        skip_grams_window_context = "right")
-      # flog.debug("tcm done dim = %d * %d", nrow(tcm), ncol(tcm))
       word_counts = attr(tcm, "word_count", TRUE)
       # cast to double in order to not get integer overflow in multiplications below
       nword = as.numeric(sum(word_counts))
@@ -183,8 +186,15 @@ Collocations = R6::R6Class(
         n_ij = as.integer(tcm@x)
       )
       # see http://www.lrec-conf.org/proceedings/lrec2002/pdf/128.pdf for details about PMI and LFMD
-      dt[ , pmi :=  log2( (n_ij / nword) / ((n_i / nword) * (n_j / nword))) ]
-      dt[ , lfmd := log2( (n_ij / nword) ^ 2 / ((n_i / nword) * (n_j / nword))) + log2(n_ij / nword)]
+      # dt[ , pmi :=  log2( (n_ij / nword) / ((n_i / nword) * (n_j / nword))) ]
+      # dt[ , lfmd := log2( (n_ij / nword) ^ 2 / ((n_i / nword) * (n_j / nword))) + log2(n_ij / nword)]
+
+      # more numerically robust approach of above expressions
+      # suggested by @andland - https://github.com/dselivanov/text2vec/issues/236
+      # derived by just openin all log2 epxressions simplifing them
+      dt[ , pmi :=  log2( n_ij ) - log2( n_i ) - log2( n_j ) + log2( nword )]
+      dt[ , lfmd := 3 * log2( n_ij ) - log2( n_i ) - log2( n_j ) - log2( nword )]
+
       # https://radimrehurek.com/gensim/models/phrases.html#gensim.models.phrases.Phrases
       # A phrase of words a and b is accepted if (cnt(a, b) - min_count) * N / (cnt(a) * cnt(b)) > threshold
       # where N is the total vocabulary size.
@@ -202,6 +212,9 @@ Collocations = R6::R6Class(
              dt$suffix %in% self$collocation_stat$suffix
       dt = dt[!dups, ]
 
+      private$iter = private$iter + 1L
+      dt[, iter := private$iter]
+
       self$collocation_stat = rbindlist(list(self$collocation_stat, dt), use.names = TRUE, fill = TRUE)
       # self$collocation_stat = self$collocation_stat[, .SD[1, ], by = .(prefix, suffix)]
       self$prune()
@@ -210,7 +223,7 @@ Collocations = R6::R6Class(
       private$phrases_ptr = create_xptr_unordered_set(private$phrases)
 
       self$collocation_stat = self$collocation_stat[order(self$collocation_stat$pmi, decreasing = TRUE)]
-
+      attr(self$collocation_stat, "nword") = nword
       invisible(self$collocation_stat)
     },
     prune = function(pmi_min = private$pmi_min, gensim_min = private$gensim_min, lfmd_min = private$lfmd_min,
@@ -234,17 +247,18 @@ Collocations = R6::R6Class(
       stopifnot(is.character(stopwords))
       stopwords_ptr = create_xptr_unordered_set(stopwords)
       collapse_collocations = function(x) {
-        collapse_collocations_cpp(x$tokens, private$phrases_ptr, stopwords_ptr, private$sep)
+        x$tokens = collapse_collocations_cpp(x$tokens, private$phrases_ptr, stopwords_ptr, private$sep)
+        x
       }
 
       if(inherits(it, "itoken_parallel")) {
-        flog.debug("clonning itoken_parallel")
+        logger$debug("clonning itoken_parallel")
         it_transformed = lapply(it, function(x) {
-          itoken_transformer_R6$new(x$clone(deep = TRUE), collapse_collocations)
+          CallbackIterator$new(x$clone(deep = TRUE), callback = collapse_collocations)
         })
         data.table::setattr(it_transformed, "class", "itoken_parallel")
       } else {
-        it_transformed = itoken_transformer_R6$new(it$clone(deep = TRUE), collapse_collocations)
+        it_transformed = CallbackIterator$new(it$clone(deep = TRUE), callback = collapse_collocations)
         data.table::setattr(it_transformed, "class", c("itoken", class(it)))
       }
       it_transformed
